#include "Arduino.h"

class Motor
{
  private:
    int pinA, pinB, pol;
  public:
    Motor(int pwmPin, int nonPwmPin, bool polarity)
    {
      pinA = pwmPin;
      pinB = nonPwmPin;
      pol = polarity > 0;

      pinMode(pinA, OUTPUT);
      pinMode(pinB, OUTPUT);

      analogWrite(pinA, 0);
      digitalWrite(pinB, 0);
    }

    void SetPower(float val)
    {
      bool dir = val > 0;
      if (val < 0)
        val *= -1;
        
      if (val > 1)
        val = 1;
      int pwm = val * 255;
      if (pwm > 255)
        pwm = 255;

      if (pwm == 0)
      {
        analogWrite(pinA, 0);
        digitalWrite(pinB, 0);
      }
      else
      {
        if (dir == pol)
        {
          analogWrite(pinA, pwm);
          digitalWrite(pinB, 0);
        }
        else
        {
          analogWrite(pinA, 255 - pwm);
          digitalWrite(pinB, 1);
        }
      }
    }
};

class MotorVectorDriver
{
  public:
    MotorVectorDriver(Motor* MotorL, Motor* MotorR)
    {
      this->MotorL = MotorL;
      this->MotorR = MotorR;
    }

    void setScale(float scl)
    {
      this->scl = scl;
      update();
    }

    void setVector(float dir, float mag)
    {
      this->dir = dir;
      this->mag = mag;
      update();
    }

    inline float GetMagnitude() { return mag; };
    inline float GetDirection() { return dir; };
    inline float GetScale() { return scl; };
    
  private:
    Motor* MotorL = NULL;
    Motor* MotorR = NULL;
    
    float mag = 0;
    float dir = 0;
    float scl = 0;

    void update() // přepočet vektoru na rychlost motorů
    {
      float pwr = mag * scl;
      
      if (pwr > 1)
        pwr = 1;
      if (pwr < 0)
        pwr = 0;
    
      if (dir > 180)
        dir = 180;
      if (dir < -180)
        dir = -180;
        
      bool forward = (-90 <= dir) && (dir <= 90);
      
      float transition;
      if (forward) // -90 to 90
      {
        transition = dir + 90; // 0 to 180
      }
      else
      {
        if (dir > 0) // 90 to 180
        {
          transition = dir - 90; // 0 to 90
        }
        else // -180 to -90
        {
          transition = dir + (180 + 90); // 90 to 180
        }
      }
      transition /= 180; // normalisation 0 to 1
    
      float left = -1 + (transition * 4);
      left = (left > 1) ? 1 : left;
    
      float right = 3 - (transition * 4);
      right = (right > 1) ? 1 : right;
    
      if (forward)
      {
        MotorL->SetPower(pwr * left);
        MotorR->SetPower(pwr * right);
      }
      else
      {
        MotorL->SetPower(pwr * (-right));
        MotorR->SetPower(pwr * (-left));
      }
    }
};
