#include <SoftwareSerial.h>
#include <limits.h>

#include "Motor.hpp" // řízení motorků
#include "Timer.hpp" // časovač
#include "LineReceiver.hpp" // příjímač

Motor motorL(5, 4, true);   // levý motor
Motor motorR(3, 2, false);  // pravý motor

MotorVectorDriver driver(&motorL, &motorR); //objekt starající se o přepočet chodu motoru

SoftwareSerial serial(7, 6);  //sériová linka pro bluetooth
LineReceiver receiver(&serial, &receiver_dataRx); //objekt pro příjem - funkce sériové linky, volaná funkce

Timer ledTimer(500, false, &ledTimer_tick); //objekt timer - interval časovače, zapnutí/vypnutí časovače, název funkce
Timer policeTimer(500, false, &policeTimer_tick); //objekt timer - interval časovače, zapnutí/vypnutí časovače, název funkce
Timer beaconTimer(100, false, &beaconTimer_tick); //objekt timer - interval časovače, zapnutí/vypnutí časovače, název funkce

#define ledPinr1 8 //proměnná pro RGB LED č.1 kanál red
#define ledPing1 9 //proměnná pro RGB LED č.1 kanál green
#define ledPinb1 10 //proměnná pro RGB LED č.1 kanál blue
#define ledPinr2 11 //proměnná pro RGB LED č.2 kanál red
#define ledPing2 12 //proměnná pro RGB LED č.2 kanál green
#define ledPinb2 13 //proměnná pro RGB LED č.2 kanál blue

int r1=0;
int g1=0;
int b1=0;
int r2=0;
int g2=0;
int b2=0;
int t=1;
int p=1;
bool ledState1 = false; //stav LED zap/vyp
bool ledState2 = false; //stav LED zap/vyp

void setup()
{
  serial.begin(9600);
  serial.setTimeout(LONG_MAX);
  pinMode(ledPinr1, OUTPUT);
  pinMode(ledPing1, OUTPUT);
  pinMode(ledPinb1, OUTPUT);
  pinMode(ledPinr2, OUTPUT);
  pinMode(ledPing2, OUTPUT);
  pinMode(ledPinb2, OUTPUT);
  //digitalWrite(ledPinr1, ledState);

}
/* Periodické volání funkce */
void ledTimer_tick()
{
  
  ledState1 = !ledState1;  //přepínač funkce
  t=t+1;
  //první přednastavená světelná signalizace
  if (t>3)
    {
    t=1;
    }
  if (ledState1==true) 
    {
      if (t==1)
        {
        digitalWrite(ledPinr1, 1);
        digitalWrite(ledPinb1, 0);
        digitalWrite(ledPing1, 0);
        digitalWrite(ledPinr2, 1);
        digitalWrite(ledPinb2, 0);
        digitalWrite(ledPing2, 0);
        }
      if (t==2)
        {
        digitalWrite(ledPinr1, 0);
        digitalWrite(ledPinb1, 1);
        digitalWrite(ledPing1, 0);
        digitalWrite(ledPinr2, 0);
        digitalWrite(ledPinb2, 1);
        digitalWrite(ledPing2, 0);
        }
      if (t==3)
        {
        digitalWrite(ledPinr1, 0);
        digitalWrite(ledPinb1, 0);
        digitalWrite(ledPing1, 1);
        digitalWrite(ledPinr2, 0);
        digitalWrite(ledPinb2, 0);
        digitalWrite(ledPing2, 1);
        }
    }
  else 
    {
      digitalWrite(ledPinr1, 0);
      digitalWrite(ledPinb1, 0);
      digitalWrite(ledPing1, 0);
      digitalWrite(ledPinr2, 0);
      digitalWrite(ledPinb2, 0);
      digitalWrite(ledPing2, 0);
    }  
    
  
}


void policeTimer_tick()
{
  ledState2 = !ledState2;  //přepínač funkce
  p=p+1;
  //druhá přednastavená světelná signalizace
  if (p>2)
    {
    p=1;
    }

  if (ledState2==true)
    {
      if (p==1)
        {
        digitalWrite(ledPinr1, 1);
        digitalWrite(ledPinb2, 1);
        digitalWrite(ledPinr2, 0);
        digitalWrite(ledPinb1, 0);
        }
      if (p==2)
        {
        digitalWrite(ledPinr1, 0);
        digitalWrite(ledPinb2, 0);
        digitalWrite(ledPinr2, 1);
        digitalWrite(ledPinb1, 1); 
        }
    }
  else
    {
    digitalWrite(ledPinr1, 0);
    digitalWrite(ledPing1, 0);
    digitalWrite(ledPinb1, 0);
    digitalWrite(ledPinr2, 0);
    digitalWrite(ledPing2, 0);
    digitalWrite(ledPinb2, 0); 
    }  
  
}
//třetí přednastavená světelná signalizace
void beaconTimer_tick()
{
  ledState1 = !ledState1;  //přepínač funkce
  
  digitalWrite(ledPinr1, ledState1); //zap/vyp LED
  digitalWrite(ledPinr2, ledState1); //zap/vyp LED
}
/* Příjem volání z transmitteru a reakce na přijaté znaky*/
void receiver_dataRx(String msg)
{
  char code = 0;
  if(msg.length() > 0)
    code = msg[0];

  switch (code)
  {
    case 'S': // stop
      driver.setVector(0, 0);
      break;
    case 'F': // front
      driver.setVector(0, 1);
      break;
    case 'G': // back
      driver.setVector(180, 1);
      break;
    case 'L': // left
      driver.setVector(-89.9, 1);
      break;
    case 'R': // right
      driver.setVector(89.9, 1);
      break;
    case 'Q': // front-left
      driver.setVector(-45, 1);
      break;
    case 'E': // front-right
      driver.setVector(45, 1);
      break;
    case 'Z': // back-left
      driver.setVector(-135, 1);
      break;
    case 'C': // back-right
      driver.setVector(135, 1);
      break;
    case 'J': // J parameter: power control
      {
        float magnitude = ((float)msg.substring(1).toInt()) / 180.0;
        if (magnitude < 0)
          magnitude = 0;
        if (magnitude > 1)
          magnitude = 1;
        driver.setScale(magnitude);
        driver.setVector(0, 0);
      }
      break;
    case 'K': // K parameter
      //msg.substring(1).toInt()
      break;
    case 'A':  //svit LED R1
      if (r1==0)
        {
        digitalWrite(ledPinr1,true);
        r1=1;
        }
      else if (r1==1)
        {
        digitalWrite(ledPinr1,false);
        r1=0;
        }      
     break;
    case 'B':  //svit LED G1
      if (g1==0)
        {
        digitalWrite(ledPing1,true);
        g1=1;
        }
      else if (g1==1)
        {
        digitalWrite(ledPing1,false);
        g1=0;
        }      
     break;
    case 'D':  //svit LED B1
      if (b1==0)
        {
        digitalWrite(ledPinb1,true);
        b1=1;
        }
      else if (b1==1)
        {
        digitalWrite(ledPinb1,false);
        b1=0;
        }      
     break;
     case 'H':  //svit LED R2
      if (r2==0)
        {
        digitalWrite(ledPinr2,true);
        r2=1;
        }
      else if (r2==1)
        {
        digitalWrite(ledPinr2,false);
        r2=0;
        }      
     break;
    case 'I':  //svit LED G2
      if (g2==0)
        {
        digitalWrite(ledPing2,true);
        g2=1;
        }
      else if (g2==1)
        {
        digitalWrite(ledPing2,false);
        g2=0;
        }      
     break;
    case 'P':  //svit LED B2
      if (b2==0)
        {
        digitalWrite(ledPinb2,true);
        b2=1;
        }
      else if (b2==1)
        {
        digitalWrite(ledPinb2,false);
        b2=0;
        }      
     break;
    case 'T':  //reset
      digitalWrite(ledPinr1,false);
      digitalWrite(ledPing1,false);
      digitalWrite(ledPinb1,false);
      digitalWrite(ledPinr2,false);
      digitalWrite(ledPing2,false);
      digitalWrite(ledPinb2,false);
      r1=0;
      g1=0;
      b1=0;
      r2=0;
      g2=0;
      b2=0;      
     break;
    case 'U': // spuštění prvního typu signalizace
      t=0;
      ledState1 = ledTimer.enabled;
      ledTimer_tick();
      ledTimer.enabled = !ledTimer.enabled;
      ledTimer.reset();
      break;
    case 'V': // spuštění druhého typu signalizace
      p=0;
      ledState2 = policeTimer.enabled;
      policeTimer_tick();
      policeTimer.enabled = !policeTimer.enabled;
      policeTimer.reset();
      break;
    case 'W': // spuštění třetího typu signalizace
      ledState1 = beaconTimer.enabled;
      beaconTimer_tick();
      beaconTimer.enabled = !beaconTimer.enabled;
      beaconTimer.reset();
      break;
    case 'Y': // nevyužito
      break;
  }
}
/* Nekonečná smyčka spouštějící objekty - funkce clk kontroluje stav a aktivuje případnou činnost */
void loop()
{
  receiver.clk();
  
  ledTimer.clk();
  policeTimer.clk();
  beaconTimer.clk();
}
